/*
 * @Descripttion: 
 * @version: 
 * @Author: guanxiaoxin
 * @Date: 2023-10-23 13:38:34
 * @LastEditors: guanxiaoxin
 * @LastEditTime: 2023-10-24 10:45:53
 * @FilePath: \VNoteApp\lib\pages\medical\devices\controllers\NIBP.dart
 */
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnote_device_plugin/devices/nibp.dart';
import 'package:vnote_device_plugin/models/exams/nibp.dart';
import 'package:vnoteapp/pages/medical/devices/interfaces/device_controller.dart';
import 'package:vnoteapp/pages/medical/models/item.dart';
import 'package:vnoteapp/pages/medical/models/worker.dart';

class NIBPController extends GetxController implements IDeviceController {
  NIBPController({
    required this.medicalItem,
    required this.deviceChange,
  });

  late final NibpDeviceWorker worker = NibpDeviceWorker(
    mac: 'A4:C1:38:33:84:F8',
    model: 'AOJ-30B',
  );
  MedicalItem medicalItem;
  ValueChanged deviceChange;

  /// 连接设备
  @override
  Future<void> connectDevice() async {
    await worker.connect();
  }

  @override
  void workerAddListener() async {
    worker.resultUpdateEvent.addListener(_onSuccess);
    worker.connectErrorEvent.addListener(_onConnectFail);
    worker.connectedEvent.addListener(_onConnectSuccess);
    worker.disconnectedEvent.addListener(_onDisconnected);
  }

  @override
  void onInit() {
    connectDevice();
    workerAddListener();
    super.onInit();
  }

  @override
  void onClose() {
    disConnectDevice();
    super.onClose();
  }

  void _onSuccess(_, NibpExamValue value) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connected;

    medicalItem.diagnosisData = jsonEncode({
      'Sbp': value.diastolicPressure ?? '0',
      'Dbp': value.systolicPressure ?? '0',
      'Pulse_Beat': value.pulse
    });
    deviceChange.call(medicalItem);
  }

  void _onConnectFail(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connectionFailed;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  void _onConnectSuccess(sender, e) {
    print(e);
  }

  void _onDisconnected(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.disconnected;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  @override
  Future<void> disConnectDevice() async {
    worker.connectErrorEvent.removeListener(_onConnectFail);
    worker.connectedEvent.removeListener(_onConnectSuccess);
    worker.resultUpdateEvent.removeListener(_onSuccess);
    worker.disconnectedEvent.removeListener(_onDisconnected);
    await worker.disconnect();
  }
}
