/*
 * @Descripttion: 
 * @version: 
 * @Author: guanxiaoxin
 * @Date: 2023-10-23 13:38:34
 * @LastEditors: guanxiaoxin
 * @LastEditTime: 2023-10-24 10:29:28
 * @FilePath: \VNoteApp\lib\pages\medical\devices\controllers\sp_o2.dart
 */
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnote_device_plugin/devices/sp_o2.dart';
import 'package:vnote_device_plugin/models/exams/sp_o2.dart';
import 'package:vnoteapp/pages/medical/devices/interfaces/device_controller.dart';
import 'package:vnoteapp/pages/medical/models/item.dart';
import 'package:vnoteapp/pages/medical/models/worker.dart';

class SpO2Controller extends GetxController implements IDeviceController {
  SpO2Controller({
    required this.medicalItem,
    required this.deviceChange,
  });
  late final SpO2DeviceWorker worker = SpO2DeviceWorker(
    mac: 'A4:C1:38:0B:CC:60',
    model: 'AOJ-70B',
  );
  MedicalItem medicalItem;
  ValueChanged deviceChange;

  /// 连接设备
  @override
  Future<void> connectDevice() async {
    await worker.connect();
  }

  @override
  void workerAddListener() async {
    worker.valueUpdateEvent.addListener(_onSuccess);
    worker.connectErrorEvent.addListener(_onConnectFail);
    worker.connectedEvent.addListener(_onConnectSuccess);
    worker.disconnectedEvent.addListener(_onDisconnected);
  }

  @override
  void onInit() {
    connectDevice();
    workerAddListener();
    super.onInit();
  }

  @override
  void onClose() {
    disConnectDevice();
    super.onClose();
  }

  void _onSuccess(_, SpO2ExamValue e) {
    print(e);
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connected;

    medicalItem.diagnosisData = jsonEncode(
        {'Pulse_Frequency': e.pulse.toString(), 'Spo2': e.spO2.toString()});
    deviceChange.call(medicalItem);
  }

  void _onConnectFail(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connectionFailed;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  void _onConnectSuccess(sender, e) {
    print(e);
  }

  void _onDisconnected(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.disconnected;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  @override
  Future<void> disConnectDevice() async {
    worker.connectErrorEvent.removeListener(_onConnectFail);
    worker.connectedEvent.removeListener(_onConnectSuccess);
    worker.valueUpdateEvent.removeListener(_onSuccess);
    worker.disconnectedEvent.removeListener(_onDisconnected);
    await worker.disconnect();
  }
}
