/*
 * @Descripttion: 
 * @version: 
 * @Author: guanxiaoxin
 * @Date: 2023-10-23 13:38:34
 * @LastEditors: guanxiaoxin
 * @LastEditTime: 2023-10-23 18:07:47
 * @FilePath: \VNoteApp\lib\pages\medical\devices\controllers\sugar.dart
 */
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnote_device_plugin/devices/sugar.dart';
import 'package:vnote_device_plugin/models/exams/sugar.dart';
import 'package:vnoteapp/pages/medical/devices/interfaces/device_controller.dart';
import 'package:vnoteapp/pages/medical/models/item.dart';
import 'package:vnoteapp/pages/medical/models/worker.dart';

class SugarController extends GetxController implements IDeviceController {
  SugarController({
    required this.medicalItem,
    required this.deviceChange,
  });
  final SugarDeviceWorker worker = SugarDeviceWorker(
    mac: '60:98:66:C4:0E:51',
    model: 'i-sens630',
  );
  MedicalItem medicalItem;
  ValueChanged deviceChange;

  /// 连接设备
  @override
  Future<void> connectDevice() async {
    await worker.connect();
  }

  @override
  void workerAddListener() async {
    worker.successEvent.addListener(_onSuccess);
    worker.connectErrorEvent.addListener(_onConnectFail);
    worker.connectedEvent.addListener(_onConnectSuccess);
    worker.disconnectedEvent.addListener(_onDisconnected);
  }

  @override
  void onInit() {
    connectDevice();
    workerAddListener();
    super.onInit();
  }

  @override
  void onClose() {
    disConnectDevice();
    super.onClose();
  }

  void _onSuccess(_, SugarExamData e) {
    print(e);
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connected;
    medicalItem.diagnosisData = jsonEncode({'sugar': e.sugar.toString()});
    deviceChange.call(medicalItem);
  }

  void _onConnectFail(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.connectionFailed;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  void _onConnectSuccess(sender, e) {
    print(e);
  }

  void _onDisconnected(sender, e) {
    medicalItem.diagnosisWorkerStatus = WorkerStatus.disconnected;
    deviceChange.call(medicalItem);
    connectDevice();
  }

  @override
  Future<void> disConnectDevice() async {
    worker.connectErrorEvent.removeListener(_onConnectFail);
    worker.connectedEvent.removeListener(_onConnectSuccess);
    worker.successEvent.removeListener(_onSuccess);
    worker.disconnectedEvent.removeListener(_onDisconnected);
    await worker.disconnect();
  }
}
