import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnoteapp/components/dialog_number.dart';
import 'package:vnoteapp/pages/check/widgets/exam_configurable/exam_card.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_disconnected.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_fail.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_success.dart';
import 'package:vnoteapp/pages/medical/controller.dart';
import 'package:vnoteapp/pages/medical/models/worker.dart';

class BodyTemperature extends GetView<MedicalController> {
  const BodyTemperature({super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      var element = controller.state.medicalMenuList
          .firstWhere((element) => element.key == 'Temp');
      return Stack(
        children: [
          _buildTemperature(),
          _buildStutas(element.diagnosisWorkerStatus),
        ],
      );
    });
  }

  Widget _buildTemperature() {
    return ExamCard(
      title: '体温',
      clickCard: () {
        _inputTemperature();
      },
      content: Container(
        alignment: Alignment.bottomRight,
        padding: const EdgeInsets.only(
          bottom: 20,
          right: 30,
          left: 40,
        ),
        constraints: const BoxConstraints(minHeight: 150),
        child: FittedBox(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.end,
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              Obx(() {
                String? value =
                    controller.state.medicalMenuList.first.diagnosisData;
                String text = value != null
                    ? jsonDecode(value)['Temperature'].toString()
                    : '';
                return RichText(
                  text: TextSpan(
                    text: text,
                    style: const TextStyle(
                      fontSize: 80,
                      color: Colors.black,
                    ),
                    children: const [
                      TextSpan(
                        text: '°C',
                        style: TextStyle(fontSize: 25),
                      )
                    ],
                  ),
                );
              })
            ],
          ),
        ),
      ),
    );
  }

  Future<void> _inputTemperature() async {
    String? result = await VDialogNumber(
      title: '体温',
      initialValue: jsonDecode(
          controller.state.medicalMenuList.first.diagnosisData ??
              '')['Temperature'],
    ).show();
    if (result?.isNotEmpty ?? false) {
      jsonDecode(controller.state.medicalMenuList.first.diagnosisData ?? '')[
          'Temperature'] = result ?? '';
    }
  }

  Widget _buildStutas(diagnosisWorkerStatus) {
    switch (diagnosisWorkerStatus) {
      case WorkerStatus.connectionFailed:
        return const DeviceConnectFail();
      case WorkerStatus.connected:
        return const DeviceConnectSuccess();
      case WorkerStatus.connecting:
        return const DeviceConnect();
      case WorkerStatus.disconnected:
        return const DeviceConnectDisconnected();
      default:
        return const SizedBox();
    }
  }
}

// class BodyTemperature extends StatefulWidget {
//   const BodyTemperature({
//     super.key,
//   });

//   @override
//   State<BodyTemperature> createState() => _ExamBodyTemperatureState();
// }

// class _ExamBodyTemperatureState extends State<BodyTemperature> {
//   var permissionManager = Get.find<IPermissionManager>();
//   var controller = Get.find<MedicalController>();

//   late final TempDeviceWorker worker = TempDeviceWorker(
//     mac: 'D0:05:00:00:00:67',
//     model: 'YHW-6',
//   );

//   /// 设备连接失败的状态
//   bool _connectFailStatus = false;

//   /// 设备连接中断的状态
//   bool _connectDisconnectedStatus = false;

//   /// 设备连接成功的状态
//   bool _connectSuccessStatus = false;

//   /// 设备是否连接中
//   bool _isConnect = false;
//   late String _value =
//       controller.diagnosisDataValue['Temp']?['Temperature'] ?? '';
//   @override
//   void initState() {
//     initTemperature();
//     super.initState();
//   }

//   Future<void> initTemperature() async {
//     getPermission();
//     workerAddListener();
//     await connect();
//     await initData();
//   }

//   void workerAddListener() {
//     worker.successEvent.addListener(_onSuccess);
//     worker.connectErrorEvent.addListener(_onConnectFail);
//     worker.connectedEvent.addListener(_onConnectSuccess);
//     worker.disconnectedEvent.addListener(_onDisconnected);
//   }

//   Future<void> initData() async {
//     await controller.readCachedRecord();
//     _value = controller.diagnosisDataValue['Temp']?['Temperature'] ?? '';
//     setState(() {});
//   }

//   Future<void> connect() async {
//     _connectFailStatus = false;
//     _isConnect = true;
//     _connectSuccessStatus = false;
//     _connectDisconnectedStatus = false;
//     await worker.connect();

//     setState(() {});
//   }

//   Future<void> disconnect() async {
//     worker.connectErrorEvent.removeListener(_onConnectFail);
//     worker.connectedEvent.removeListener(_onConnectSuccess);
//     worker.successEvent.removeListener(_onSuccess);
//     worker.disconnectedEvent.removeListener(_onDisconnected);
//     await worker.disconnect();
//   }

//   @override
//   void dispose() {
//     disconnect();
//     super.dispose();
//   }

//   void _onSuccess(_, double e) {
//     setState(() {
//       _value = e.toString();
//       controller.diagnosisDataValue['Temp'] = {'Temperature': _value};
//       _isConnect = false;
//     });
//   }

//   void _onConnectFail(sender, e) {
//     print('连接设备失败');
//     _connectFailStatus = true;
//     _connectSuccessStatus = false;
//     _connectDisconnectedStatus = false;
//     _isConnect = false;
//     setState(() {});
//   }

//   void _onConnectSuccess(sender, e) {
//     _connectSuccessStatus = true;
//     _isConnect = false;
//     _connectFailStatus = false;
//     _connectDisconnectedStatus = false;
//     setState(() {});
//   }

//   void _onDisconnected(sender, e) {
//     print('设备连接中断');
//     _connectDisconnectedStatus = true;
//     _connectSuccessStatus = false;
//     _connectFailStatus = false;
//     _isConnect = false;
//     setState(() {});
//   }

//   Future<void> getPermission() async {
//     await permissionManager.requestLocationPermission();
//     await permissionManager.requestBluetoothConnectPermission();
//     await permissionManager.requestBluetoothAdvertisePermission();
//     await permissionManager.requestBluetoothScanPermission();
//   }

//   @override
//   Widget build(BuildContext context) {
    // return Stack(
    //   children: [
    //     _buildTemperature(),
    //     if (_connectFailStatus)
    //       DeviceConnectFail(
    //         connect: () => connect(),
    //       ),
    //     if (_connectSuccessStatus) const DeviceConnectSuccess(),
    //     if (_isConnect) const DeviceConnect(),
    //     if (_connectDisconnectedStatus)
    //       DeviceConnectDisconnected(
    //         connect: () => connect(),
    //       ),
    //   ],
    // );
//   }

//   Widget _buildTemperature() {
//     return ExamCard(
//       title: '体温',
//       clickCard: () {
//         _inputTemperature();
//       },
//       content: Container(
//         alignment: Alignment.bottomRight,
//         padding: const EdgeInsets.only(
//           bottom: 20,
//           right: 30,
//           left: 40,
//         ),
//         constraints: const BoxConstraints(minHeight: 150),
//         child: FittedBox(
//           child: Row(
//             mainAxisAlignment: MainAxisAlignment.end,
//             crossAxisAlignment: CrossAxisAlignment.end,
//             children: [
//               RichText(
//                 text: TextSpan(
//                   text: _value,
//                   style: const TextStyle(
//                     fontSize: 80,
//                     color: Colors.black,
//                   ),
//                   children: const [
//                     TextSpan(
//                       text: '°C',
//                       style: TextStyle(fontSize: 25),
//                     )
//                   ],
//                 ),
//               ),
//             ],
//           ),
//         ),
//       ),
//     );
//   }

//   Future<void> _inputTemperature() async {
//     String? result = await VDialogNumber(
//       title: '体温',
//       initialValue: _value,
//     ).show();
//     if (result?.isNotEmpty ?? false) {
//       _value = result ?? '';
//     }
//   }
// }
