import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vnoteapp/components/dialog_number.dart';
import 'package:vnoteapp/pages/check/widgets/exam_configurable/exam_card.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_disconnected.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_fail.dart';
import 'package:vnoteapp/pages/check/widgets/exam_device_connect_status/connect_success.dart';
import 'package:vnoteapp/pages/medical/controller.dart';
import 'package:vnoteapp/pages/medical/models/worker.dart';

class BloodOxygen extends GetView<MedicalController> {
  const BloodOxygen({super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      var element = controller.state.medicalMenuList
          .firstWhere((element) => element.key == 'SpO2');
      return Stack(
        children: [
          _buildTemperature(),
          _buildStutas(element.diagnosisWorkerStatus),
        ],
      );
    });
  }

  Widget _buildTemperature() {
    return ExamCard(
      title: '血氧',
      clickCard: () {
        _inputTemperature();
      },
      content: Container(
        alignment: Alignment.bottomRight,
        padding: const EdgeInsets.only(
          bottom: 20,
          right: 30,
          left: 40,
        ),
        constraints: const BoxConstraints(minHeight: 150),
        child: FittedBox(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.end,
            crossAxisAlignment: CrossAxisAlignment.end,
            children: [
              Obx(() {
                var element = controller.state.medicalMenuList
                    .firstWhere((element) => element.key == 'SpO2');
                String? value = element.diagnosisData;
                String text =
                    value != null ? jsonDecode(value)['sugar'].toString() : '';
                return RichText(
                  text: TextSpan(
                    text: text,
                    style: const TextStyle(
                      fontSize: 80,
                      color: Colors.black,
                    ),
                    children: const [
                      TextSpan(
                        text: 'mmol/L',
                        style: TextStyle(fontSize: 25),
                      )
                    ],
                  ),
                );
              })
            ],
          ),
        ),
      ),
    );
  }

  Future<void> _inputTemperature() async {
    String? result = await VDialogNumber(
      title: '血糖',
      initialValue: jsonDecode(
          controller.state.medicalMenuList.first.diagnosisData ?? '')['sugar'],
    ).show();
    if (result?.isNotEmpty ?? false) {
      jsonDecode(controller.state.medicalMenuList.first.diagnosisData ?? '')[
          'sugar'] = result ?? '';
    }
  }

  Widget _buildStutas(diagnosisWorkerStatus) {
    switch (diagnosisWorkerStatus) {
      case WorkerStatus.connectionFailed:
        return const DeviceConnectFail();
      case WorkerStatus.connected:
        return const DeviceConnectSuccess();
      case WorkerStatus.connecting:
        return const DeviceConnect();
      case WorkerStatus.disconnected:
        return const DeviceConnectDisconnected();
      default:
        return const SizedBox();
    }
  }
}



// ignore: must_be_immutable
// class BloodOxygen extends StatefulWidget {
//   const BloodOxygen({
//     super.key,
//   });

//   @override
//   State<BloodOxygen> createState() => _ExamBloodOxygenState();
// }

// class _ExamBloodOxygenState extends State<BloodOxygen> {
//   var permissionManager = Get.find<IPermissionManager>();
//   var controller = Get.find<MedicalController>();

//   late final SpO2DeviceWorker worker = SpO2DeviceWorker(
//     mac: 'A4:C1:38:0B:CC:60',
//     model: 'AOJ-70B',
//   );

//   late String pulse =
//       controller.diagnosisDataValue['SpO2']?['Pulse_Frequency'] ?? '0';
//   late String spO2 = controller.diagnosisDataValue['SpO2']?['Spo2'] ?? '0';
//   bool _connectFailStatus = false;
//   bool _connectSuccessStatus = false;
//   bool _isConnect = false;

//   /// 设备连接中断的状态
//   bool _connectDisconnectedStatus = false;
//   @override
//   void initState() {
//     initOxygen();

//     super.initState();
//   }

//   void workerAddListener() {
//     worker.valueUpdateEvent.addListener(_onSuccess);
//     worker.connectErrorEvent.addListener(_onConnectFail);
//     worker.connectedEvent.addListener(_onConnectSuccess);
//   }

//   Future<void> initOxygen() async {
//     getPermission();
//     workerAddListener();
//     await connect();
//     await initData();
//   }

//   Future<void> initData() async {
//     await controller.readCachedRecord();
//     pulse = controller.diagnosisDataValue['SpO2']?['Pulse_Frequency'];
//     spO2 = controller.diagnosisDataValue['SpO2']?['Spo2'];
//     setState(() {});
//   }

//   Future<void> connect() async {
//     _connectFailStatus = false;
//     _isConnect = true;
//     _connectSuccessStatus = false;
//     setState(() {});
//     await worker.connect();
//   }

//   Future<void> disconnect() async {
//     worker.connectErrorEvent.removeListener(_onConnectFail);
//     worker.connectedEvent.removeListener(_onConnectSuccess);
//     worker.valueUpdateEvent.removeListener(_onSuccess);
//     worker.disconnectedEvent.removeListener(_onDisconnected);
//     await worker.disconnect();
//   }

//   @override
//   void dispose() {
//     disconnect();
//     super.dispose();
//   }

//   void _onSuccess(_, SpO2ExamValue e) {
//     setState(() {
//       pulse = e.pulse.toString();
//       spO2 = e.spO2.toString();
//       controller.diagnosisDataValue['SpO2'] = {
//         'Pulse_Frequency': pulse,
//         'Spo2': spO2
//       };
//       _isConnect = false;
//     });
//   }

//   void _onConnectFail(sender, e) {
//     print('连接设备失败');
//     _connectFailStatus = true;
//     _connectSuccessStatus = false;
//     _isConnect = false;
//     _connectDisconnectedStatus = false;
//     setState(() {});
//   }

//   void _onConnectSuccess(sender, e) {
//     _connectSuccessStatus = true;
//     _connectFailStatus = false;
//     _isConnect = false;
//     _connectDisconnectedStatus = false;
//     setState(() {});
//   }

//   void _onDisconnected(sender, e) {
//     print('设备连接中断');
//     _connectDisconnectedStatus = true;
//     _connectSuccessStatus = false;
//     _connectFailStatus = false;
//     _isConnect = false;
//     setState(() {});
//   }

//   Future<void> getPermission() async {
//     await permissionManager.requestLocationPermission();
//     await permissionManager.requestBluetoothConnectPermission();
//     await permissionManager.requestBluetoothAdvertisePermission();
//     await permissionManager.requestBluetoothScanPermission();
//   }

//   @override
//   Widget build(BuildContext context) {
//     return Stack(
//       children: [
//         ExamCard(
//             // clickCard: () {},
//             content: Column(
//           mainAxisAlignment: MainAxisAlignment.start,
//           children: [
//             SideBar(
//               title: '血氧饱和度',
//               value: spO2,
//               unit: '%',
//             ),
//             const Divider(indent: 30),
//             SideBar(
//               title: '脉率',
//               value: pulse,
//               unit: 'bpm',
//             ),
//           ],
//         )),
//         if (_connectFailStatus)
//           DeviceConnectFail(
//             connect: () => connect(),
//           ),
//         if (_connectSuccessStatus) const DeviceConnectSuccess(),
//         if (_isConnect) const DeviceConnect(),
//         if (_connectDisconnectedStatus)
//           DeviceConnectDisconnected(
//             connect: () => connect(),
//           ),
//       ],
//     );
//   }
// }
